# coding=utf-8


def calc_distance(dhash1, dhash2):
    difference = (int(dhash1, 16)) ^ (int(dhash2, 16))
    return bin(difference).count("1")


if __name__ == '__main__':

    dhash_dict = {}
    for line in open("dhash.result.2018_06_07_14"):
        line = line.strip()
        dhash_dict[line.split(",")[0]] = line.split(",")[1]

    result_dict = {}
    for key0, value0 in dhash_dict.items():
        for key1, value1 in dhash_dict.items():
            if key0 == key1:
                continue
            distance = calc_distance(value0, value1)
            result_dict["%s,%s" % (key0, key1)] = distance

    sorted_burst_dict = sorted(result_dict.iteritems(), key=lambda d: d[1], reverse=False)

    result = open("java_distance.csv", "w+")

    for item in sorted_burst_dict:
        result.write("%s,%s\n" % (item[0], item[1]))

    result.close()



# coding=utf-8
import requests
import re

from geopy import Nominatim
from pydawn.string_utils import gen_md5

from pydawn.file_utils import open_file_in_utf8
import matplotlib
from matplotlib.collections import PatchCollection
from matplotlib.font_manager import FontProperties
from mpl_toolkits.basemap import Basemap
import matplotlib.pyplot as plt
from matplotlib.patches import Polygon
import time
import os
import numpy as np
import sys
from peewee import *

from pydawn.sqlite_utils import sqlite_get_db, sqlite_excute

reload(sys)
sys.setdefaultencoding('utf-8')

location_cache = "location_cache.txt"
base_dir = os.path.dirname(__file__).replace("\\", "/")
font = FontProperties(fname="%s/fonts/simsun.ttc" % base_dir, size=14)

db = SqliteDatabase('locations.db')


def refine_address(address):
    address = address.encode("utf-8").lower()
    pattern = re.compile("\s+")
    return pattern.sub(" ", address).strip()


class BaseModel(Model):
    class Meta:
        database = db


class Address(BaseModel):
    address = CharField(unique=True)
    address_md5 = CharField(unique=True)
    longitude = FloatField()
    latitude = FloatField()


def insert_address(address, longitude, latitude):
    address = refine_address(address)
    md5 = gen_md5(address)
    try:
        address_object = Address.get(Address.address_md5 == md5)
        if address_object.longitude is None:
            address_object.longitude = float(longitude)
            address_object.latitude = float(latitude)
            address_object.save()
    except:
        address_object = Address.create(address=address,
                                        address_md5=md5,
                                        longitude=float(longitude),
                                        latitude=float(latitude))

        address_object.save()


def get_cached_coordinate(address):
    address = refine_address(address)
    md5 = gen_md5(address)
    try:
        address_object = Address.get(Address.address_md5 == md5)
        if address_object.longitude is not None:
            return address_object.longitude, address_object.latitude
    except:
        pass

    return None, None


def init_address_db():
    db.connect()
    db.create_tables([Address])


def get_coordinate(location):
    url_format = "http://api.map.baidu.com/geocoder/v2/?output=json&ak=SjDhGSaC0GTQfhL7ezS9Qb0MoTWk49hO&address=%s"
    url = url_format % location
    response = requests.get(url, timeout=8)
    answer = response.json()
    try:
        if answer["status"] == 1:
            return get_coordinates3(location)
        x, y = answer['result']['location']['lng'], answer['result']['location']['lat']
        print "get coordinate for %s(%s,%s)" % (location, x, y)
        return x, y
    except:
        print 'query location %s fail, %s' % (location, answer)
        return get_coordinates3(location)


def get_coordinates2(location):
    url_format = "http://restapi.amap.com/v3/geocode/geo?key=ea2b07318489822d536cf4d90c436522&address=%s"
    url = url_format % location
    response = requests.get(url)
    answer = response.json()
    print answer


def get_coordinates3(location):
    geolocator = Nominatim()
    try:
        loc = geolocator.geocode(location, timeout=20)
        print "get_coordinates3: %s(%s,%s)" % (location, loc.longitude, loc.latitude)
        return loc.longitude, loc.latitude
    except:
        return None, None


def draw_cn_map(location_file):
    fig = plt.figure(figsize=(16, 8))
    m = Basemap(llcrnrlon=77, llcrnrlat=14, urcrnrlon=140, urcrnrlat=51, projection='lcc', lat_1=33, lat_2=45, lon_0=100)
    m.drawcoastlines()
    m.drawcountries(linewidth=1.5)

    plt.gca().xaxis.set_major_locator(plt.NullLocator())
    plt.gca().yaxis.set_major_locator(plt.NullLocator())
    plt.subplots_adjust(top=1, bottom=0, right=1, left=0, hspace=0, wspace=0)
    plt.margins(0, 0)

    ax = plt.gca()
    #ax.spines['top'].set_visible(False)
    #ax.spines['bottom'].set_visible(False)
    #ax.spines['left'].set_visible(False)
    #ax.spines['right'].set_visible(False)

    #plt.axis('off')

    base_dir = os.path.dirname(__file__).replace("\\", "/")
    m.readshapefile('%s/CHN_adm_shp/CHN_adm1' % base_dir, 'states', drawbounds=True)
    ax = plt.gca()
    for nshape, seg in enumerate(m.states):
        poly = Polygon(seg, facecolor='#96CDCD')
        ax.add_patch(poly)

    m.readshapefile('%s/TWN_adm_shp/TWN_adm0' % base_dir, 'taiwan', drawbounds=True)
    for nshape, seg in enumerate(m.taiwan):
        poly = Polygon(seg, facecolor='#96CDCD')
        ax.add_patch(poly)

    for line in open(location_file):
        location = re.split(",\s;", line)[0]
        location = location.replace(";", "").strip()

        longitude, latitude = get_cached_coordinate(location)
        if longitude is None:
            continue

        x, y = m(longitude, latitude)
        m.plot(x, y, marker='o', color='r', markersize=3, alpha=0.8, zorder=10)

    province = []
    for shapedict in m.states_info:
        statename = shapedict['NL_NAME_1']
        p = statename.split('|')
        if len(p) > 1:
            s = p[1]
        else:
            s = p[0]

        if s not in province:
            province.append(s)

    for shapedict in m.taiwan_info:
        s = shapedict['NAME_CHINE']
        if s not in province:
            province.append(s)

    province.append(u"黑龙江")
    font = FontProperties(fname="%s/fonts/simsun.ttc" % base_dir, size=14)
    for pro in province:
        print "pro", pro
        try:
            if pro == u"海南" or pro == u"河南":
                pro = pro + u"省"

            longitude, latitude = get_cached_coordinate(pro)
            if longitude is None:
                continue

            x, y = m(longitude, latitude)
            plt.text(x, y, pro, fontsize=8, color='#000000', zorder=100, fontproperties=font)
            print "add text %s" % pro
        except:
            pass

    start_x, start_y = m(127.500830295, 50.2506900907)
    end_x, end_y = m(98.4999883215, 25.2799781235)
    plt.plot([start_x, end_x], [start_y, end_y], color="g", linestyle='--', linewidth=3)
    fig.savefig("cn_map.jpg", format='jpg', dpi=300, pad_inches=0)
    plt.show()


def draw_label(x, y, width, height, ax, color, text):
    rect_pos = np.zeros(shape=(4, 2))
    rect_pos[0][0] = x
    rect_pos[0][1] = y

    rect_pos[1][0] = x
    rect_pos[1][1] = y - height

    rect_pos[2][0] = x + width
    rect_pos[2][1] = y - height

    rect_pos[3][0] = x + width
    rect_pos[3][1] = y

    patches = []
    polygon = Polygon(rect_pos, True)
    patches.append(polygon)
    p = PatchCollection(patches, cmap=matplotlib.cm.jet, alpha=0.4, facecolors=color, edgecolors="#000000")
    ax.add_collection(p)

    text_margin = 10000
    top_margin = 1000
    plt.text(x+width+text_margin, y-top_margin, text, fontsize=14, color='#000000', zorder=100, fontproperties=font, ha='left', va="top")


def draw_cities(city_list, save_path):

    coords = []
    for city in city_list:
        print repr(city)
        x, y = location_coord[city]
        coords.append((x, y))

    padding = 0.05
    llcrnrlon = min(coords,key=lambda x:x[0])[0] * (1-padding)
    urcrnrlon = max(coords, key=lambda x: x[0])[0] * (1+padding)
    llcrnrlat = min(coords, key=lambda x: x[1])[1] * (1-padding)
    urcrnrlat = max(coords, key=lambda x: x[1])[1] * (1+padding)

    mid_lon = (llcrnrlon+urcrnrlon) / 2
    mid_lat = (llcrnrlat+urcrnrlat) / 2

    show_label = True
    start = time.clock()

    plt.figure(figsize=(20, 20))
    m = Basemap(llcrnrlon=llcrnrlon, urcrnrlon=urcrnrlon, llcrnrlat=llcrnrlat, urcrnrlat=urcrnrlat, projection='lcc', lon_0=mid_lon, lat_0=mid_lat)

    m.readshapefile("CHN_adm_shp\\CHN_adm2", 'states', drawbounds=False)
    m.drawcoastlines(color="w", zorder=-1)

    ax = plt.gca()
    # ax.spines['top'].set_visible(False)
    # ax.spines['bottom'].set_visible(False)
    # ax.spines['left'].set_visible(False)
    # ax.spines['right'].set_visible(False)


    color1 = "#ffff00"
    color2 = "#def9de"
    color3 = "#bbfbba"
    color4 = "#9eeb1a"
    color5 = "#00FF00"

    for info, shp in zip(m.states_info, m.states):
        province = info['NAME_1']
        city = info['NAME_2']
        city_name = info["NL_NAME_2"].split("|")[0].encode("utf-8")
        if city_name in city_list:
            color = 'g'
            poly = Polygon(shp, facecolor=color, edgecolor='#000000', lw=1)
            ax.add_patch(poly)

            if show_label:
                x, y = location_coord[city_name]
                x, y = m(x, y)
                # print city_name, x, y
                city_name = city_name.replace(u"襄樊", u"襄阳")
                plt.text(x, y, city_name, fontsize=10, color='#000000', zorder=100, fontproperties=font, ha='center')

    # draw arrow
    '''
    x, y = location_coord["arrow"]
    x, y = m(x, y)
    plt.text(x, y, "N", fontsize=14, color='#000000', zorder=100, fontproperties=font, ha='center')

    x_offset = 25000
    y_offset = 70000
    height = 50000

    left_angle = np.zeros(shape=(3, 2))
    left_angle[0][0] = x
    left_angle[0][1] = y

    left_angle[1][0] = x - x_offset
    left_angle[1][1] = y - y_offset

    left_angle[2][0] = x
    left_angle[2][1] = y - height

    left_patches = []
    left_polygon = Polygon(left_angle, True)
    left_patches.append(left_polygon)

    left_p = PatchCollection(left_patches, cmap=matplotlib.cm.jet, alpha=0.4, facecolors="#000000",
                             edgecolors="#000000")
    ax.add_collection(left_p)

    right_angle = np.zeros(shape=(3, 2))
    right_angle[0][0] = x
    right_angle[0][1] = y

    right_angle[1][0] = x + x_offset
    right_angle[1][1] = y - y_offset

    right_angle[2][0] = x
    right_angle[2][1] = y - height

    right_patches = []
    right_polygon = Polygon(right_angle, True, color="w")
    right_patches.append(right_polygon)

    right_p = PatchCollection(right_patches, cmap=matplotlib.cm.jet, alpha=0.4, facecolors="w", edgecolors="#000000")
    ax.add_collection(right_p)

    # draw label
    width = 50000
    height = 20000
    location = location_coord["label"]
    x = float(location.split(",")[0])
    y = float(location.split(",")[1])
    x, y = m(x, y)

    margin = 10000
    draw_label(x, y, width, height, ax, color1, "<0.2")
    draw_label(x, y - (height + margin), width, height, ax, color2, "0.2～0.4")
    draw_label(x, y - (height + margin) * 2, width, height, ax, color3, "0.4～0.6")
    draw_label(x, y - (height + margin) * 3, width, height, ax, color4, "0.6～0.8")
    draw_label(x, y - (height + margin) * 4, width, height, ax, color5, ">0.8")
    '''

    end = time.clock()
    print(end - start)
    plt.savefig(save_path)
    plt.show()


def get_coordinates(location_list_file):
    for line in open(location_list_file):
        address = refine_address(line)
        longitude, latitude = get_cached_coordinate(address)
        if longitude is None:
            longitude, latitude = get_coordinate(address)
            if longitude is not None:
                insert_address(address, longitude, latitude)


def draw_world_map(location_list_file):
    fig = plt.figure(figsize=(20, 10))
    world_map = Basemap(projection='cyl')
    world_map.drawmapboundary()
    world_map.drawcountries(linewidth=0.25)
    world_map.fillcontinents(color='lightgray', zorder=1)

    plt.gca().xaxis.set_major_locator(plt.NullLocator())
    plt.gca().yaxis.set_major_locator(plt.NullLocator())
    plt.subplots_adjust(top=1, bottom=0, right=1, left=0, hspace=0, wspace=0)
    plt.margins(0, 0)

    # draw country name first
    country_list = "country.csv"
    get_coordinates(country_list)

    for line in open(country_list):
        country = line.strip()

        longitude, latitude = get_cached_coordinate(country)
        if longitude is None:
            continue

        x, y = world_map(longitude, latitude)
        plt.text(x, y, country, fontsize=6, color='#8B8378', zorder=100)

    for line in open(location_list_file):
        address = refine_address(line)
        longitude, latitude = get_cached_coordinate(address)
        if longitude is None:
            continue
        x, y = world_map(longitude, latitude)
        world_map.plot(x, y, marker='o', markersize=7, color='Red', alpha=0.3, zorder=10)

    fig.savefig("world_map.jpg", format='jpg', dpi=300, pad_inches=0)
    plt.show()


if __name__ == '__main__':
    init_address_db()
    #draw_cn_map("author_location_cn.txt")
    '''
    for line in open("location_cache_refine.txt"):
        address = line.split("|")[0]
        longitude = line.split("|")[1].split(" ")[0]
        latitude = line.split("|")[1].split(" ")[1]
        insert_address(address, longitude, latitude)
    '''

    #str = "van godewijckstraat 30  netherlands"
    #refine_cache()
    #get_coordinates("author_location_en.txt")
    draw_world_map("author_location_en.txt")
    #get_coordinates2(u"华中科技大学")
    #get_coordinates2("VAN GODEWIJCKSTRAAT 30, 3311 GZ DORDRECHT, NETHERLANDS")



# coding=utf-8
import hashlib
import re
from urlparse import urlparse

import chardet


def gen_md5(text):
    m = hashlib.md5()
    m.update(text)
    return m.hexdigest()


def refine_urls(host, seed_url, urls):
    urls = sorted(list(set(urls)))
    refined_urls = []
    seed_uri = urlparse(seed_url)
    for url in urls:
        uri = urlparse(url)
        if len(uri.netloc) == 0:
            path = seed_uri.path + url
            path = path.replace("//", "/")
            url = seed_uri.scheme + "://" + seed_uri.netloc + path
        elif url.find(host) == -1:
            continue

        refined_urls.append(url.strip())

    refined_urls = sorted(list(set(refined_urls)))
    return refined_urls


def get_links(host, seed_url, html):
    url_regex = re.compile('<a[^>]+href=["\'](.*?)["\']', re.IGNORECASE)
    return refine_urls(host, seed_url, url_regex.findall(html))


def detect_str_charset(content):
    result = chardet.detect(content)
    encoding = result['encoding']
    if encoding is None:
        encoding = "utf-8"
    return encoding.lower()


def convert_string_to_utf8(content):
    char_set = detect_str_charset(content)
    content = content.decode(char_set, errors="ignore").encode("utf-8")
    return content


def pick_charset(html):
    charset = None
    m = re.compile('<meta .*(http-equiv="?Content-Type"?.*)?charset="?([a-zA-Z0-9_-]+)"?', re.I).search(html)
    if m and m.lastindex == 2:
        charset = m.group(2).lower()
    return charset


if __name__ == '__main__':
    content = open("test.html").read()
    content = pick_charset(content)
    print content




# coding=utf-8
import os
from PIL import Image

def dhash(image, hash_size = 8):
    # Grayscale and shrink the image in one step.
    image = image.convert('L').resize(
                    (hash_size + 1, hash_size),
                    Image.LANCZOS,
                    )
    #pixels = list(image.getdata())

    # Compare adjacent pixels.
    difference = []
    for row in xrange(hash_size):
        for col in xrange(hash_size):
            pixel_left = image.getpixel((col, row))
            pixel_right = image.getpixel((col + 1, row))
            difference.append(pixel_left > pixel_right)
    # Convert the binary array to a hexadecimal string.
    decimal_value = 0
    hex_string = []
    for index, value in enumerate(difference):
        if value:
            decimal_value += 2**(index % 8)
        if (index % 8) == 7:
            hex_string.append(hex(decimal_value)[2:].rjust(2, '0'))
            decimal_value = 0
    return ''.join(hex_string)


def calc_distance(dhash1, dhash2):
    difference = (int(dhash1, 16)) ^ (int(dhash2, 16))
    return bin(difference).count("1")


if __name__ == '__main__':
    dhash_1 = str(dhash(Image.open("D:\\ai\\fake_img\\0F7B8658FDC98DD64DC5018D8C32BF03.png")))
    dhash_2 = str(dhash(Image.open("D:\\ai\\fake_img\\499DC96BE6167DED3580605434CACF33.png")))
    print dhash_1
    print dhash_2
    print calc_distance(dhash_1, dhash_2)

    '''
    img_dir = "D:\\ai\\fake_img\\"
    dhash_dict = {}
    for file_name in os.listdir(img_dir):
        file_path = os.path.join(img_dir, file_name)
        dhash_value = str(dhash(Image.open(file_path)))
        dhash_dict[file_name] = dhash_value

    result_dict = {}
    for key0, value0 in dhash_dict.items():
        for key1, value1 in dhash_dict.items():
            if key0 == key1:
                continue
            distance = calc_distance(value0, value1)
            result_dict["%s,%s" % (key0, key1)] = distance

    sorted_burst_dict = sorted(result_dict.iteritems(), key=lambda d: d[1], reverse=False)

    result = open("python_distance_bi_raw.csv", "w+")

    for item in sorted_burst_dict:
        result.write("%s,%s\n" % (item[0], item[1]))

    result.close()
    '''



# coding=utf-8
import hashlib
import os
import chardet
import codecs


def get_file_md5(filename):
    if not os.path.isfile(filename):
        return
    file_hash = hashlib.md5()
    f = file(filename, 'rb')
    while True:
        b = f.read(8096)
        if not b :
            break
        file_hash.update(b)
    f.close()
    return file_hash.hexdigest().upper()


def detect_file_charset(file_path):
    f = open(file_path, "rb")
    test_length = 1024
    data = f.read(test_length)
    result = chardet.detect(data)
    encoding = result['encoding']
    if encoding is None:
        encoding = "utf-8"
    return encoding.lower()


def open_file_in_utf8(file_path):
    encoding = detect_file_charset(file_path)
    f = codecs.open(file_path, encoding=encoding)
    return f


def list_file_recursively(sample_dir):
    g = os.walk(sample_dir)
    for path, d, file_list in g:
        for file_name in file_list:
            yield os.path.join(path, file_name)


if __name__ == '__main__':
    pass




# coding=utf-8
import logging
import datetime
import os
from command import CommandRunner
import re
import time


class AdbUtil:
    def __init__(self, device_id="", logger_name=None):
        self.device_id = device_id
        self.logger = logging.getLogger(logger_name)

    def run_command(self, command, timeout=180):
        print command
        self.logger.info("run command: %s, timeout:%d" % (command, timeout))
        cmd = CommandRunner(command, timeout)
        code, output = cmd.run()
        return output

    @staticmethod
    def list_devices():
        command = "adb devices"
        cmd = CommandRunner(command, 180)
        code, output = cmd.run()
        devices = output.split("\n")
        device_list = []
        for device in devices[1:]:
            device_id = device.replace("device", "").strip()
            if len(device_id) == 0:
                continue
            device_list.append(device_id)

        return device_list

    def adb_command(self, command, timeout=180):
        if len(self.device_id) != 0:
            cmd = "adb -s %s %s" % (self.device_id, command)
        else:
            cmd = "adb %s" % command
        return self.run_command(cmd, timeout)

    def keep_app_alive(self, package):
        running_apps = self.get_running_apps()
        if package not in running_apps:
            self.start_app(package)

    def set_phone_time(self):
        now_time = datetime.datetime.now().strftime('%m%d%H%M%Y.%S')
        command = "shell date %s" % now_time
        self.adb_command(command)

    def prepare_phone_for_test(self):
        # root
        self.adb_command("root")
        # 最大化休眠时间
        self.adb_command("shell settings put system screen_off_timeout 1800000")
        # 取消时区自动获取
        self.adb_command("shell settings put global auto_time 0")
        # 接电不息屏
        self.adb_command("shell settings put global stay_on_while_plugged_in 3")
        # 禁止旋转
        self.adb_command("shell settings put system user_rotation 0")
        # 24小时制
        self.adb_command("shell settings put system time_12_24 24")
        # 设置当前时间
        self.set_phone_time()

    def start_app(self, package):
        self.adb_command("shell monkey -p %s -c android.intent.category.LAUNCHER 1" % package)

    def install_app(self, apk_path):
        self.adb_command("install -r %s" % apk_path)

    def monkey_app(self, package, times):
        self.adb_command("shell monkey -p %s %d" % (package, times))

    def uninstall_app(self, package):
        self.adb_command("uninstall %s" % package)
        self.adb_command("shell rm -fr /data/app/%s*" % package)

    def stop_app(self, package):
        running_apps = self.get_running_apps()
        if package in running_apps:
            self.adb_command("shell am force-stop %s" % package)

    def get_packages(self, white_list=[]):
        command = "shell pm list package -3"
        content = self.adb_command(command)

        package_list = []
        for line in content.split("\n"):
            try:
                package = line.split(":")[1].strip()

                is_white = 0
                for white_app in white_list:
                    if package.find(white_app) != -1:
                        is_white = 1
                        break
                if is_white == 1:
                    continue

                package_list.append(package)
            except:
                pass
        return package_list

    def clean_all_app(self, white_list):
        package_list = self.get_packages(white_list)
        for package in package_list:
            self.stop_app(package)
            self.uninstall_app(package)

    def enable_wifi(self):
        self.adb_command("shell svc wifi enable")

    def get_apk_package(self, apk_path):
        base_dir = os.path.dirname(os.path.abspath(__file__))
        aapt_path = os.path.join(base_dir, "aapt.exe")
        name_pattern = re.compile(".*name=\'(.*?)\'.*")

        # get package name
        aapt_command = "%s d badging %s" % (aapt_path, apk_path)
        content = self.run_command(aapt_command)
        for line in content.split("\n"):
            match = name_pattern.match(line)
            if match:
                return match.group(1)

        return None

    def disable_airplane_model(self):
        self.adb_command("shell settings put global airplane_mode_on 0")
        self.adb_command("shell am broadcast -a android.intent.action.AIRPLANE_MODE")

    def hide_status_bar(self):
        self.adb_command("settings put global policy_control immersive.full=*")

    def reboot_device(self, password):
        self.adb_command("reboot")
        while True:
            device_online = self.device_id in self.list_devices()
            if device_online:
                break
            time.sleep(5)
            print "waiting for %s bootup..." % self.device_id

        print "device %s bootup success..." % self.device_id
        time.sleep(15)
        self.adb_command("shell input tap 1178 1452")
        self.adb_command("shell input text %s" % password)
        self.adb_command("shell input keyevent 66")
        time.sleep(15)
        self.adb_command("shell input tap 1178 1452")

    def check_if_screen_on(self):
        content = self.adb_command("shell dumpsys window policy")
        for line in content.split("\n"):
            if line.find("mScreenOnEarly") != -1:
                if line.find("mScreenOnEarly=false") != -1:
                    return False
                if line.find("mScreenOnEarly=true") != -1:
                    return True

    def check_if_screen_lock(self):
        content = self.adb_command("shell dumpsys window policy")
        for line in content.split("\n"):
            if line.find("isStatusBarKeyguard") != -1:
                if line.find("isStatusBarKeyguard=false") != -1:
                    return False
                if line.find("isStatusBarKeyguard=true") != -1:
                    return True

    def unlock_device(self, password):
        if self.check_if_screen_on():
            self.adb_command("shell input keyevent 26")
            time.sleep(2)
        self.adb_command("shell input keyevent 26")
        if not self.check_if_screen_lock():
            return
        # adb_command(device_id, "shell input keyevent 66")
        self.adb_command("shell input keyevent 82")
        self.adb_command("shell input text %s" % password)
        self.adb_command("shell input keyevent 66")

    def retrieve_data(self):
        pattern = re.compile("\d{4}_\d{2}_\d{2}_\d{2}.csv")
        command = "shell ls /sdcard/"
        output = self.adb_command(command)
        base_dir = os.path.dirname(os.path.abspath(__file__))
        for file_name in output.split("\n"):
            file_name = file_name.strip().split(".csv")[0] + ".csv"
            match = pattern.match(file_name)
            if match:
                command = "pull /sdcard/%s %s" % (file_name, "%s/result/%s_%s" % (base_dir, self.device_id, file_name))
                self.adb_command(command)

    def get_running_apps(self):
        command = "shell ps -A"
        output = self.adb_command(command)
        pattern = re.compile("\s+")
        running_apps = []
        for line in output.split("\n"):
            line = line.strip()
            if len(line) == 0:
                continue
            parts = pattern.split(line)
            if not parts[0].startswith("u0"):
                continue

            if parts[-1].find(".") == -1:
                continue

            package = parts[-1].split(":")[0]
            running_apps.append(package)

        return running_apps

    def get_crash_packages(self):
        output = self.adb_command("logcat -s AndroidRuntime -t 10000")
        pattern = re.compile(".*AndroidRuntime: Process: (.*), PID:.*")
        crash_packages = []
        for line in output.split("\n"):
            match = pattern.match(line)
            if match:
                crash_package = match.group(1).strip()
                crash_package = crash_package.split(":")[0].strip()
                crash_packages.append(crash_package)

        return crash_packages, output


if __name__ == "__main__":
    white_list = ["com.coloros.safecenter", "com.antiy"]
    sample_dir = "Z:\\ai\\sample\\blackmail_float"
    logger_name = "auto_test"
    logging.basicConfig(level=logging.INFO,
                        name=logger_name,
                        format='%(asctime)s %(filename)s: %(message)s',
                        datefmt='%a,%d %b %Y %H:%M:%S',
                        filename='%s/auto_test.%s' % (sample_dir, time.strftime('%Y%m%d', time.localtime(time.time()))),
                        filemode='a+')
    adb_util = AdbUtil()
    print adb_util.get_running_apps()





# coding=utf-8

import numpy as np
import burst_detection as bd

import sys
reload(sys)
sys.setdefaultencoding('utf-8')


def calculate_yaccord_matrix(keyword_count_file, keywords_file, tag="", delimiter="|"):
    top_keywords = []
    top_keywords_dict = {}
    top_keywords_count_dict = {}
    top_keywords_num = 60

    co_occur_array = np.zeros(shape=(top_keywords_num, top_keywords_num))
    count = 0
    for line in open(keyword_count_file):

        keyword = line.split(delimiter)[0]
        top_keywords_count_dict[keyword] = int(line.split(delimiter)[1])
        top_keywords.append(keyword)
        top_keywords_dict[keyword] = count

        count += 1
        if count >= top_keywords_num:
            break

    # load keywords wo count co-occurrence
    for line in open(keywords_file):
        keywords = line.split(delimiter)
        for keyword_x in keywords[:-1]:
            for keyword_y in keywords:
                if keyword_x == keyword_y:
                    continue
                try:
                    co_occur_array[top_keywords_dict[keyword_x]][top_keywords_dict[keyword_y]] += 1
                except:
                    pass

    co_occur_freq = open("co_occur_freq_%s.csv" % tag, "w+")
    for top_keyword in top_keywords:
        co_occur_freq.write(top_keyword+",")

    co_occur_freq.write("\n")

    for line in co_occur_array:
        for ele in line:
            co_occur_freq.write("%d," % ele)
        co_occur_freq.write("\n")

    co_occur_freq.close()

    # calculate yaccord matrix
    yaccord_co_occur_array = np.zeros(shape=(top_keywords_num, top_keywords_num), dtype=np.float64)
    for keyword_x in top_keywords:
        for keyword_y in top_keywords:
            if keyword_x == keyword_y:
                continue

            row = top_keywords_dict[keyword_x]
            col = top_keywords_dict[keyword_y]

            row_count = top_keywords_count_dict[keyword_x]
            col_count = top_keywords_count_dict[keyword_y]

            value = 1.0 * co_occur_array[row][col]/(0.001 + row_count+col_count-co_occur_array[row][col])
            yaccord_co_occur_array[row][col] = value

    yaccord_co_occur_array_freq = open("yaccord_co_occur_array_freq_%s.csv" % tag, "w+")
    yaccord_co_occur_array_freq.write(",")
    for idx, top_keyword in enumerate(top_keywords):
        if idx != len(line) - 1:
            yaccord_co_occur_array_freq.write(top_keyword + ",")
        else:
            yaccord_co_occur_array_freq.write(top_keyword)

    yaccord_co_occur_array_freq.write("\n")

    for index, line in enumerate(yaccord_co_occur_array):
        yaccord_co_occur_array_freq.write(top_keywords[index] + ",")
        for idx, ele in enumerate(line):
            ele = ele
            if idx != len(line) - 1:
                yaccord_co_occur_array_freq.write("%f," % ele)
            else:
                yaccord_co_occur_array_freq.write("%f" % ele)
        yaccord_co_occur_array_freq.write("\n")

    yaccord_co_occur_array_freq.close()


if __name__ == '__main__':
    keyword_count_file = "keyword_count.csv"
    keywords_file = "keywords.txt"
    calculate_yaccord_matrix(keyword_count_file, keywords_file, "test")



# coding=utf-8

import burst_detection as bd
import re
import matplotlib.pyplot as pl
import numpy as np
import matplotlib as mpl
from matplotlib.ticker import MultipleLocator, FuncFormatter
import sys

from pydawn.co_occurrence import calculate_yaccord_matrix

reload(sys)
sys.setdefaultencoding('utf-8')


def burst_detect(r, d, label, gamma=1):
    # number of time points
    n = len(r)

    # find the optimal state sequence (q)
    q, d, r, p = bd.burst_detection(r, d, n, s=3, gamma=gamma, smooth_win=1)

    # enumerate bursts based on the optimal state sequence
    bursts = bd.enumerate_bursts(q, label)

    # find weight of bursts
    weighted_bursts = bd.burst_weights(bursts, r, d, p)

    if len(weighted_bursts) > 0:
        print weighted_bursts


def detect_burst_words(start_year, end_year, keywords_with_year_file, keyword_count_file, top_keywords_num=60, tag="", gamma=1, delimiter="|"):
    ori_stdout = sys.stdout
    burst_raw_file = "burst_result_raw_%s.txt" % tag
    f = open(burst_raw_file, 'w+')
    sys.stdout = f

    year_range = end_year - start_year + 1

    top_keywords = []
    top_keywords_dict = {}
    top_keywords_count_dict = {}

    count = 0
    for line in open(keyword_count_file):
        keyword = line.split(delimiter)[0]
        top_keywords_count_dict[keyword] = int(line.split(delimiter)[1])
        top_keywords.append(keyword)
        top_keywords_dict[keyword] = count

        count += 1
        if count >= top_keywords_num:
            break

    keywords_freq_by_year = np.zeros(shape=(top_keywords_num, year_range))
    for line in open(keywords_with_year_file):
        keywords = line.split(delimiter)
        year = int(keywords[-1]) - start_year
        for index, keyword in enumerate(keywords):
            if index != len(keywords)-1 and keyword in top_keywords:
                keywords_freq_by_year[top_keywords_dict[keyword]][year] += 1

    all_keyword_sum = np.sum(keywords_freq_by_year, axis=0)

    for index, year_count in enumerate(keywords_freq_by_year):
        keyword = top_keywords[index]
        try:
            burst_detect(year_count, all_keyword_sum, keyword, gamma)
        except:
            #print "%s burst exception" % keyword
            pass

    f.close()
    sys.stdout = ori_stdout

    burst_file = open("burst_detection_%s.csv" % tag, "w+")

    burst_dict = {}
    for line in open(burst_raw_file):
        line = line.strip()
        if len(line) == 0 or line.find("label") != -1:
            continue

        parts = re.split("\s*", line)
        begin = int(parts[2]) + start_year
        end = int(parts[3]) + start_year
        line = "%s,%s,%s,%s\n" % (parts[1], parts[2], parts[3], parts[4])
        burst_dict[line] = begin

    sorted_burst_dict = sorted(burst_dict.iteritems(), key=lambda d: d[0], reverse=False)
    for item in sorted_burst_dict:
        print item[0]
        burst_file.write(item[0])

    burst_file.close()


def draw_burst(start_year, end_year, tag=""):
    burst_file = "burst_detection_%s.csv" % tag

    pl.figure(figsize=(10, 6))
    ax = pl.gca()
    mpl.rcParams['font.sans-serif'] = ['SimHei']
    pl.subplots_adjust(bottom=0.15)
    pl.grid()

    keyword_list = []
    for index, line in enumerate(open(burst_file)):
        parts = line.split(",")
        if parts[0].strip() not in keyword_list:
            keyword_list.append(parts[0].strip().decode("utf-8"))

    pl.axis([start_year, end_year + 1, -1, len(keyword_list)])

    def keyword_format(index, pos):
        index = int(index)

        if index < 0 or index > len(keyword_list)-1:
            return ""
        return keyword_list[index]

    ax.xaxis.set_major_locator(MultipleLocator(1))
    ax.yaxis.set_major_formatter(FuncFormatter(keyword_format))
    ax.yaxis.set_major_locator(MultipleLocator(1))

    for line in open(burst_file):
        parts = line.split(",")

        weight = parts[3].strip()

        start = int(parts[1]) + start_year
        end = int(parts[2]) + start_year

        x = np.arange(start, end + 1, 1)
        y = np.empty(end - start + 1)
        index = keyword_list.index(parts[0].strip())
        y[:] = index
        pl.plot(x, y, linewidth=4, color="green")

        if weight != '0':
            pl.text(x[0], y[0], weight)

        if start == end:
            pl.scatter(start, index, color="green")

    pl.show()


if __name__ == '__main__':
    tag = "ssci"
    start = 2000
    end = 2017
    keyword_count_file = "keywords_count_%s.csv" % tag
    keywords_file = "keywords_%s.txt" % tag
    keywords_with_year_file = "keywords_with_year_%s.csv" % tag
    detect_burst_words(start, end, keywords_with_year_file, keyword_count_file, tag=tag, gamma=1)
    draw_burst(start, end, tag)
    calculate_yaccord_matrix(keyword_count_file, keywords_with_year_file, tag)




# coding=utf-8
import rsa
from Crypto.Cipher import AES
from Crypto import Random
import binascii

buffer_size = 1024 * 1024 * 16


class RsaUtil:

    def __init__(self, public_key_file=None, private_key_file=None):
        self.max_buffer_room = 245
        self.rsa_block_size = 256
        if public_key_file is not None:
            self.load_rsa_pub_key(public_key_file)
        else:
            self.pub_key = None

        if private_key_file is not None:
            self.load_rsa_pri_key(private_key_file)
        else:
            self.pri_key = None

    def gen_rsa_key_pair(self, public_key_file, private_key_file):
        (self.pub_key, self.pri_key) = rsa.newkeys(2048)
        with open(public_key_file, 'w+') as f:
            f.write(self.pub_key.save_pkcs1().decode())

        with open(private_key_file, 'w+') as f:
            f.write(self.pri_key.save_pkcs1().decode())

    def load_rsa_pub_key(self, public_key_file):
        with open(public_key_file, 'r') as f:
            self.pub_key = rsa.PublicKey.load_pkcs1(f.read().encode())

    def load_rsa_pri_key(self, private_key_file):
        with open(private_key_file, 'r') as f:
            self.pri_key = rsa.PrivateKey.load_pkcs1(f.read().encode())

    def rsa_encrypt_string(self, msg):
        return rsa.encrypt(msg, self.pub_key)

    def rsa_decrypt_string(self, msg):
        return rsa.decrypt(msg, self.pri_key)

    def rsa_encrypt_file(self, input_file_path, output_file_path):
        output_file = open(output_file_path, "wb+")
        with open(input_file_path, "rb") as f:
            file_buffer = f.read(self.max_buffer_room)
            while file_buffer != "":
                content = self.rsa_encrypt_string(file_buffer)
                output_file.write(content)
                file_buffer = f.read(self.max_buffer_room)

        output_file.close()

    def rsa_decrypt_file(self, input_file_path, output_file_path):
        output_file = open(output_file_path, "wb+")
        with open(input_file_path, "rb") as f:
            file_buffer = f.read(self.rsa_block_size)
            while file_buffer != "":
                content = self.rsa_decrypt_string(file_buffer)
                output_file.write(content)
                file_buffer = f.read(self.rsa_block_size)

        output_file.close()


class AesUtil:

    def __init__(self, aes_key_path=None):
        if aes_key_path is not None:
            self.load_aes_key_iv(aes_key_path)
        else:
            self.key = None
            self.iv = None

    def gen_aes_key_iv(self, aes_key_path):
        self.key = Random.new().read(AES.block_size)
        self.iv = Random.new().read(AES.block_size)
        ase_file = open(aes_key_path, "wb+")
        ase_file.write(self.key)
        ase_file.write(self.iv)
        ase_file.close()

    def load_aes_key_iv(self, aes_key_path):
        ase_file = open(aes_key_path, "rb")
        self.key = ase_file.read(AES.block_size)
        self.iv = ase_file.read(AES.block_size)
        ase_file.close()

    def aes_encrypt_string(self, msg):
        cipher = AES.new(self.key, AES.MODE_CBC, self.iv)
        mode = len(msg) % AES.block_size
        msg = msg + " " * (AES.block_size - mode)
        return cipher.encrypt(msg)

    def aes_decrypt_string(self, msg):
        cipher = AES.new(self.key, AES.MODE_CBC, self.iv)
        return cipher.decrypt(msg).strip()

    def aes_encrypt_file(self, input_path, out_path):
        encrypt_file = open(out_path, "wb+")
        with open(input_path, "rb") as f:
            file_buffer = f.read(buffer_size)
            while file_buffer != "":
                buffer_enc = self.aes_encrypt_string(file_buffer)
                encrypt_file.write(buffer_enc)
                file_buffer = f.read(buffer_size)

        encrypt_file.close()

    def aes_decrypt_file(self, input_path, out_path):
        decrypt_file = open(out_path, "wb+")
        with open(input_path, "rb") as f:
            file_buffer = f.read(buffer_size)
            while file_buffer != "":
                buffer_dec = self.aes_decrypt_string(file_buffer)
                decrypt_file.write(buffer_dec)
                file_buffer = f.read(buffer_size)

        decrypt_file.close()


if __name__ == '__main__':
    # gen_rsa_key_pair("zc.pub", "zc.pri")
    # aes_file_path = "zc.aes"
    # aes_util = AesUtil(aes_file_path)
    # aes_util.aes_encrypt_file("zc.rar", "zc.rar.enc")
    # aes_util.aes_decrypt_file("zc.rar.enc", "zc.rar.enc.rar")

    pub_key_file = "test.pub"
    pri_key_file = "test.pri"
    rsa_util = RsaUtil(pub_key_file, pri_key_file)

    ori_str = "zhangchao"
    str_encrypt = rsa_util.rsa_encrypt_string(ori_str)
    str_decrypt = rsa_util.rsa_decrypt_string(str_encrypt)
    print str_decrypt

    rsa_util.rsa_encrypt_file("pydawn.zip", "pydawn.zip.rsa.enc")
    rsa_util.rsa_decrypt_file("pydawn.zip.rsa.enc", "pydawn.rar.rsa.dec.zip")
